@@ -4,43 +4,50 @@ import serial
 from time import sleep
 
 
-NUM_KEY_DIGITS = 32
-SERIAL_SPEEDS  = [1, 2, 4, 9, 38, 57, 115, 230, 460, 1000]
-AIR_SPEEDS     = [12, 56, 64, 100, 125, 200, 224, 500, 750]
-ANT_MODES      = [1, 2, 3]
-NET_IDS        = list(range(500))
-TX_PWRS        = list(range(31))
-MIN_FREQS      = list(range(895000, 936000, 1000))
-MAX_FREQS      = list(range(895000, 936000, 1000))
-NUM_CHANNELS   = list(range(1, 51))
-DUTY_CYCLES    = list(range(10, 110, 10))
-LBT_RSSIS      = [0, 25, 50, 100, 150, 200, 220]
-MAX_WINDOWS    = list(range(33, 132))
-COMMANDS       = {'+++':       'Enter AT mode',
-                  'ATI':       'Shows the radio version',
-                  'ATI2':      'Shows the board type',
-                  'ATI3':      'Shows board frequency',
-                  'ATI4':      'Shows board version',
-                  'ATI5':      'Shows all user settable EEPROM parameters and their values',
-                  'ATI5?':     'Shows all user settable EEPROM parameters and their possible range',
-                  'ATI6':      'Displays TDM timing report',
-                  'ATI7':      'Displays RSSI signal report',
-                  'ATI8':      'Display Device 64 bit unique ID',
-                  'ATI9':      'Display node ID [multipoint only]',
-                  'ATO':       'Exits AT command mode',
-                  'AT{n}?':    'Displays radio parameter number ‘n’',
-                  'AT{n}={X}': 'Sets radio parameter number ‘n’ to ‘X’',
-                  'ATZ':       'Reboots the radio',
-                  'AT&W':      'Writes current parameters to EEPROM',
-                  'AT&F':      'Resets all parameters to factory defaults',
-                  'AT&T=RSSI': 'Enables RSSI debugging report',
-                  'AT&T=TDM':  'Enables TDM debugging report',
-                  'AT&UPDATE': 'Reset and enter boot mode',
-                  'AT&T':      'Disables debugging report'}
+NUM_KEY_DIGITS      = 32
+SERIAL_SPEEDS       = [1, 2, 4, 9, 19, 38, 57, 115, 230, 460, 1000]
+SERIAL_SPEEDS_TRUE  = [1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200, 230400, 460800, 1000000]
+AIR_SPEEDS          = [12, 56, 64, 100, 125, 200, 250, 500, 750]
+ANT_MODES           = [1, 2, 3]
+NET_IDS             = list(range(500))
+TX_PWRS             = list(range(31))
+MIN_FREQS           = list(range(895000, 936000, 1000))
+MAX_FREQS           = list(range(895000, 936000, 1000))
+NUM_CHANNELS        = list(range(1, 51))
+DUTY_CYCLES         = list(range(10, 110, 10))
+LBT_RSSIS           = [0, 25, 50, 100, 150, 200, 220]
+TARGET_RSSIS        = list(range(50, 256))
+HYSTERESIS_RSSIS    = list(range(20, 51))
+MAX_WINDOWS         = list(range(33, 132))
+RATE_AND_FREQ_BANDS = list(range(4))
+ANT_MODES           = list(range(4))
+MAVLINK_MODES       = list(range(3))
+MAVLINK_MODES_TRUE  = ['Raw Data', 'Mavlink', 'Low Latency']
+COMMANDS            = {'+++':       'Enter AT mode',
+                       'ATI':       'Shows the radio version',
+                       'ATI2':      'Shows the board type',
+                       'ATI3':      'Shows board frequency',
+                       'ATI4':      'Shows board version',
+                       'ATI5':      'Shows all user settable EEPROM parameters and their values',
+                       'ATI5?':     'Shows all user settable EEPROM parameters and their possible range',
+                       'ATI6':      'Displays TDM timing report',
+                       'ATI7':      'Displays RSSI signal report',
+                       'ATI8':      'Display Device 64 bit unique ID',
+                       'ATI9':      'Display node ID [multipoint only]',
+                       'ATO':       'Exits AT command mode',
+                       'AT{n}?':    'Displays radio parameter number ‘n’',
+                       'AT{n}={X}': 'Sets radio parameter number ‘n’ to ‘X’',
+                       'ATZ':       'Reboots the radio',
+                       'AT&W':      'Writes current parameters to EEPROM',
+                       'AT&F':      'Resets all parameters to factory defaults',
+                       'AT&T=RSSI': 'Enables RSSI debugging report',
+                       'AT&T=TDM':  'Enables TDM debugging report',
+                       'AT&UPDATE': 'Reset and enter boot mode',
+                       'AT&T':      'Disables debugging report'}
 
 
 def gen_key(num_digits=NUM_KEY_DIGITS):
-    return ''.join([string.hexdigits[random.randint(0, len(string.hexdigits))].upper() for num in range(num_digits)])
+    return ''.join([string.hexdigits[random.randint(0, len(string.hexdigits) - 1)].upper() for num in range(num_digits)])
 
 
 class RFDConfig(object):
@@ -49,9 +56,6 @@ class RFDConfig(object):
         self.port.timeout = 2
         self.params = {'radioVersion':    {'id':              'I',
                                            'description':     'Radio version',
-                                           'defaultVal':      None,
-                                           'minVal':          None,
-                                           'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -59,9 +63,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': False},
                        'boardType':       {'id':              'I2',
                                            'description':     'Board type',
-                                           'defaultVal':      None,
-                                           'minVal':          None,
-                                           'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -69,9 +70,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': False},
                        'boardFreq':       {'id':              'I3',
                                            'description':     'Board frequency',
-                                           'defaultVal':      None,
-                                           'minVal':          None,
-                                           'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -79,49 +77,62 @@ class RFDConfig(object):
                                            'sameOnAllModems': False},
                        'boardVersion':    {'id':              'I4',
                                            'description':     'Board version',
-                                           'defaultVal':      None,
-                                           'minVal':          None,
-                                           'maxVal':          None,
+                                           'curVal':          None,
+                                           'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
+                                           'sameOnAllModems': False},
+                       'tdmTimingReport': {'id':              'I6',
+                                           'description':     'TDM Timing Report',
+                                           'curVal':          None,
+                                           'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
+                                           'sameOnAllModems': False},
+                       'rssiSignalReport':{'id':              'I7',
+                                           'description':     'RSSI signal report',
+                                           'curVal':          None,
+                                           'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
+                                           'sameOnAllModems': False},
+                       'deviceUniqueID':  {'id':              'I8',
+                                           'description':     'Device 64 bit unique ID',
+                                           'curVal':          None,
+                                           'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
+                                           'sameOnAllModems': False},
+                       'nodeID':          {'id':              'I9',
+                                           'description':     'node ID [multipoint only]',
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
                                            'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'FORMAT':          {'id':              'S0',
-                                           'description':     'This is for EEPROM version, can’t be changed',
-                                           'defaultVal':      'firmware V',
-                                           'minVal':          None,
-                                           'maxVal':          None,
+                                           'description':     'This is for EEPROM version, it should not be changed. It is set by the firmware',
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
                                            'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'SERIAL_SPEED':    {'id':              'S1',
-                                           'description':     'Serial speed in ‘one byte form’',
-                                           'defaultVal':      57,
-                                           'minVal':          2,
-                                           'maxVal':          115,
+                                           'description':     'Serial speed in ‘one byte form’. Accepted values are 1, 2, 4, 9, 19, 38, 57, 115, 230, 460 and 1000 corresponding to 1200bps, 2400bps, 4800bps, 9600bps, 19200bps, 38400bps, 57600bps, 115200bps, 230400bps, 460800bps and 1000000bps respectively',
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
                                            'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'AIR_SPEED':       {'id':              'S2',
-                                           'description':     'Air data rate in one byte form',
-                                           'defaultVal':      128,
-                                           'minVal':          2,
-                                           'maxVal':          250,
+                                           'description':     'Air data rate in ‘one byte form’. Accepted values are 12, 56, 64, 100, 125, 200, 250, 500 and 750 corresponding to 12000bps, 56000bps, 64000bps, 100000bps, 125000bps, 200000bps, 250000bps, 500000bps and 750000bps respectively',
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
                                            'desValRemote':    None,
                                            'sameOnAllModems': True},
                        'NETID':           {'id':              'S3',
-                                           'description':     'Network ID. It should be the same on both modems',
-                                           'defaultVal':      25,
-                                           'minVal':          0,
-                                           'maxVal':          499,
+                                           'description':     'Network ID. The same on both modems in the pair',
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -129,19 +140,13 @@ class RFDConfig(object):
                                            'sameOnAllModems': True},
                        'TXPOWER':         {'id':              'S4',
                                            'description':     'Transmit power in dBm. Maximum is 30dBm',
-                                           'defaultVal':      27,
-                                           'minVal':          0,
-                                           'maxVal':          30,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
                                            'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'ECC':             {'id':              'S5',
-                                           'description':     'Enables or disables the golay error correcting code',
-                                           'defaultVal':      0,
-                                           'minVal':          0,
-                                           'maxVal':          1,
+                                           'description':     'Enables or disables the Golay error correcting code. When enabled, it doubles the over the air data usage',
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -149,19 +154,13 @@ class RFDConfig(object):
                                            'sameOnAllModems': True},
                        'MAVLINK':         {'id':              'S6',
                                            'description':     'Enables or disables the MAVLink framing and reporting',
-                                           'defaultVal':      0,
-                                           'minVal':          0,
-                                           'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
                                            'desValRemote':    None,
                                            'sameOnAllModems': False},
-                       'OP_RESEND':       {'id':              'S7',
-                                           'description':     'Opportunic Resend',
-                                           'defaultVal':      0,
-                                           'minVal':          0,
-                                           'maxVal':          1,
+                       'OPPRESEND':       {'id':              'S7',
+                                           'description':     'Opportunistic resend allows the node to resend packets if it has spare bandwidth',
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -169,9 +168,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': False},
                        'MIN_FREQ':        {'id':              'S8',
                                            'description':     'Min freq in KHz',
-                                           'defaultVal':      915000,
-                                           'minVal':          902000,
-                                           'maxVal':          927000,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -179,9 +175,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': True},
                        'MAX_FREQ':        {'id':              'S9',
                                            'description':     'Max freq in KHz',
-                                           'defaultVal':      928000,
-                                           'minVal':          903000,
-                                           'maxVal':          928000,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -189,9 +182,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': True},
                        'NUM_CHANNELS':    {'id':              'S10',
                                            'description':     'Number of frequency hopping channels',
-                                           'defaultVal':      20,
-                                           'minVal':          5,
-                                           'maxVal':          50,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -199,49 +189,41 @@ class RFDConfig(object):
                                            'sameOnAllModems': True},
                        'DUTY_CYCLE':      {'id':              'S11',
                                            'description':     'The percentage of time to allow transmit',
-                                           'defaultVal':      100,
-                                           'minVal':          10,
-                                           'maxVal':          100,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
                                            'desValRemote':    None,
                                            'sameOnAllModems': False},
-                       'LBT_RSSI':        {'id':              'S13',
+                       'LBT_RSSI':        {'id':              'S12',
                                            'description':     'Listen before talk threshold (This parameter shouldn’t be changed)',
-                                           'defaultVal':      0,
-                                           'minVal':          0,
-                                           'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
                                            'desValRemote':    None,
                                            'sameOnAllModems': True},
-                       'RTSCTS':          {'id':              'S13',
+                       'MANCHESTER':      {'id':              'S13',
+                                           'description':     'Manchester encoding (This parameter shouldn’t be changed)',
+                                           'curVal':          None,
+                                           'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
+                                           'sameOnAllModems': False},
+                       'RTSCTS':          {'id':              'S14',
                                            'description':     'Ready-to-send and Clear-to-send flow control',
-                                           'defaultVal':      0,
-                                           'minVal':          0,
-                                           'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
                                            'desValRemote':    None,
                                            'sameOnAllModems': False},
-                       'MAX_WINDOW':      {'id':              'S14',
+                       'MAX_WINDOW':      {'id':              'S15',
                                            'description':     'Max transit window size used to limit max time/latency if required otherwise will be set automatically',
-                                           'defaultVal':      131,
-                                           'minVal':          20,
-                                           'maxVal':          400,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
                                            'desValRemote':    None,
                                            'sameOnAllModems': False},
-                       'ENCRYPTION_LEVEL':{'id':              'S15',
+                       'ENCRYPTION_LEVEL':{'id':              'S16',
                                            'description':     'Encryption level 0=off, 1=128bit AES',
-                                           'defaultVal':      0,
-                                           'minVal':          0,
-                                           'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -249,9 +231,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': True},
                        'GPI1_1R/CIN':     {'id':              'S16',
                                            'description':     'Set GPIO 1.1 (pin 15) as R/C(PPM) input',
-                                           'defaultVal':      0,
-                                           'minVal':          0,
-                                           'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -259,9 +238,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': False},
                        'GPO1_1R/COUT':    {'id':              'S17',
                                            'description':     'Set GPIO 1.1 (pin 15) as R/C(PPM) output',
-                                           'defaultVal':      0,
-                                           'minVal':          0,
-                                           'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -269,9 +245,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': False},
                        'GPO1_1SBUSIN':    {'id':              'S18',
                                            'description':     'Set GPIO 1.1 (pin 12) as SBUS input',
-                                           'defaultVal':      0,
-                                           'minVal':          0,
-                                           'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -279,9 +252,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': False},
                        'GPO1_1SBUSOUT':    {'id':             'S19',
                                            'description':     'Set GPIO 1.1 (pin 12) as SBUS output',
-                                           'defaultVal':      0,
-                                           'minVal':          0,
-                                           'maxVal':          4,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -289,9 +259,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': False},
                        'ANT_MODE':        {'id':              'S20',
                                            'description':     '0= Diversity, 1= Antenna 1 only, 2= Antenna 2 only, 3= Antenna 1 TX and antenna 2 RX',
-                                           'defaultVal':      0,
-                                           'minVal':          0,
-                                           'maxVal':          3,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -299,9 +266,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': False},
                        'GPO1_3STATLED ':  {'id':              'S21',
                                            'description':     'Set GPIO 1.1 (pin 12) as output with state that mirrors the status LED on the modem',
-                                           'defaultVal':      0,
-                                           'minVal':          0,
-                                           'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -309,9 +273,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': False},
                        'GPO1_0TXEN485':   {'id':              'S22',
                                            'description':     'Set GPIO 1.0 (pin 13) as control signal on DINIO and RS485 interface boards.',
-                                           'defaultVal':      0,
-                                           'minVal':          0,
-                                           'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -319,9 +280,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': False},
                        'RATE/FREQBAND':   {'id':              'S23',
                                            'description':     'Changes the frequencies bands and airspeeds within set ranges on compliant modems ensuring compliance is maintained',
-                                           'defaultVal':      0,
-                                           'minVal':          0,
-                                           'maxVal':          3,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -329,9 +287,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': True},
                        'TARGET_RSSI':     {'id':              'R0',
                                            'description':     'Optimal RSSI value to try to sustain (off = 255)',
-                                           'defaultVal':      255,
-                                           'minVal':          50,
-                                           'maxVal':          255,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -339,19 +294,13 @@ class RFDConfig(object):
                                            'sameOnAllModems': False},
                        'HYSTERESIS_RSSI': {'id':              'R1',
                                            'description':     'Amount of change before power levels altered',
-                                           'defaultVal':      50,
-                                           'minVal':          20,
-                                           'maxVal':          50,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
                                            'desValRemote':    None,
                                            'sameOnAllModems': False},
-                       'EncryptionKey':   {'id':              'E',
+                       'EncryptionKey':   {'id':              '&E',
                                            'description':     'AES encryption key',
-                                           'defaultVal':      None,
-                                           'minVal':          None,
-                                           'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -359,9 +308,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': None},
                        'Print':           {'id':              'PP',
                                            'description':     'Print all Pin Settings',
-                                           'defaultVal':      None,
-                                           'minVal':          None,
-                                           'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -369,9 +315,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': None},
                        'Input':           {'id':              'PI={x}',
                                            'description':     'Set Pin x to Input',
-                                           'defaultVal':      None,
-                                           'minVal':          None,
-                                           'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -379,9 +322,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': None},
                        'Read':            {'id':              'PR={x}',
                                            'description':     'Read Pin X value (When set to input)',
-                                           'defaultVal':      None,
-                                           'minVal':          None,
-                                           'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -389,9 +329,6 @@ class RFDConfig(object):
                                            'sameOnAllModems': None},
                        'Output':          {'id':              'PO={x}',
                                            'description':     'Set Pin x to Output (Default) can only be controlled by ATPC',
-                                           'defaultVal':      None,
-                                           'minVal':          None,
-                                           'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
@@ -399,24 +336,18 @@ class RFDConfig(object):
                                            'sameOnAllModems': None},
                        'ControlOn':       {'id':              'PC={x},1',
                                            'description':     'Turn pin x on -­‐ Output Mode / Set internal pull up resistor -­‐ Input Mode',
-                                           'defaultVal':      None,
-                                           'minVal':          None,
-                                           'maxVal':          None,
                                            'curValRemote':    None,
                                            'desValRemote':    None,
                                            'sameOnAllModems': None},
                        'ControlOff':      {'id':              'PC={x},0',
                                            'description':     'Turn pin x off -­‐ Output Mode / Set internal pull down resistor -­‐ Input Mode',
-                                           'defaultVal':      None,
-                                           'minVal':          None,
-                                           'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
                                            'curValRemote':    None,
                                            'desValRemote':    None,
                                            'sameOnAllModems': None}}
     
-    def send(self, command, timeout=0.01):
+    def send(self, command, timeout=0.1):
         '''
         TODO
         '''
@@ -451,10 +382,8 @@ class RFDConfig(object):
         '''
         
         if len(response.split()) >= 2:
-            if not response.split()[1].upper() == 'ERROR':
+            if (not response.split()[1].upper() == 'ERROR') and (not '?' in response):
                 return True
-            else:
-                return False
         return False
     
     def flush(self):
@@ -536,6 +465,10 @@ class RFDConfig(object):
             self.port.close()
     
     def loadParam(self, param, local=True):
+        '''
+        TODO
+        '''
+        
         if self.port.isOpen():
             if param in self.params.keys():
                 if '=' in self.params[param]['id']:
@@ -556,7 +489,7 @@ class RFDConfig(object):
                     response = response.split('\r\n')[1:-1]
                     response = '\n'.join(response)
                     
-                    if response.upper() == 'ERROR':
+                    if 'ERROR' in response.upper():
                         if local:
                             self.params[param]['curVal'] = None
                         else:
@@ -572,9 +505,8 @@ class RFDConfig(object):
                                 self.params[param]['curVal'] = response
                             else:
                                 self.params[param]['curValRemote'] = response
-                    return True
-                else:
-                    return False
+                        return True
+        return False
     
     def loadAll(self, local=True):
         '''
